Package: TreeSummarizedExperiment
Type: Package
Title: TreeSummarizedExperiment: a S4 Class for Data with Tree
        Structures
Version: 2.18.0
Date: 2025-03-10
Authors@R: 
    c(person("Ruizhu", "Huang", email = "ruizhuRH@gmail.com",
      role = c("aut", "cre"), comment = c(ORCID = "0000-0003-3285-1945")),
      person("Felix G.M.", "Ernst", email = "felix.gm.ernst@outlook.com",
      role = c("ctb"), comment = c(ORCID = "0000-0001-5064-0928")),
      person(given = "Mark", family = "Robinson", role = c("ctb"),
             email = "mark.robinson@mls.uzh.ch",
             comment = c(ORCID = "0000-0002-3048-5518")),
      person(given = "Tuomas", family = "Borman", role = c("ctb"),
             comment = c(ORCID = "0000-0002-8563-8884")))
Description: TreeSummarizedExperiment has extended SingleCellExperiment to 
    include hierarchical information on the rows or columns of the rectangular 
    data.
Depends: R(>= 3.6.0), SingleCellExperiment, S4Vectors (>= 0.23.18),
        Biostrings
License: GPL (>=2)
Encoding: UTF-8
LazyData: true
biocViews: DataRepresentation, Infrastructure
Imports: methods, BiocGenerics, utils, ape, rlang, dplyr,
        SummarizedExperiment, BiocParallel, IRanges, treeio
VignetteBuilder: knitr
Suggests: ggtree, ggplot2, BiocStyle, knitr, rmarkdown, testthat
RoxygenNote: 7.1.1
Collate: 'TreeSummarizedExperiment.R' 'aboutLoop.R' 'allClass.R'
        'aggTSE.R' 'allGenerics.R' 'changeTree.R' 'classAccessor.R'
        'classValid.R' 'coercion.R' 'combine.R' 'data.R'
        'deprecate_Fun.R' 'internal_utils.R' 'makeTSE.R'
        'tree_addLabel.R' 'tree_asLeaf.R' 'tree_asPhylo.R'
        'tree_convertNode.R' 'tree_countLeaf.R' 'tree_countNode.R'
        'tree_distNode.R' 'tree_findAncestor.R' 'tree_findChild.R'
        'tree_findDescendant.R' 'tree_findSibling.R' 'tree_isLeaf.R'
        'tree_joinNode.R' 'tree_matTree.R' 'tree_printNode.R'
        'tree_shareNode.R' 'tree_showNode.R' 'tree_toTree.R'
        'tree_trackNode.R' 'tree_unionLeaf.R' 'updateObject.R'
git_url: https://git.bioconductor.org/packages/TreeSummarizedExperiment
git_branch: RELEASE_3_22
git_last_commit: f463a1d
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-31 00:37:34 UTC; biocbuild
Author: Ruizhu Huang [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3285-1945>),
  Felix G.M. Ernst [ctb] (ORCID: <https://orcid.org/0000-0001-5064-0928>),
  Mark Robinson [ctb] (ORCID: <https://orcid.org/0000-0002-3048-5518>),
  Tuomas Borman [ctb] (ORCID: <https://orcid.org/0000-0002-8563-8884>)
Maintainer: Ruizhu Huang <ruizhuRH@gmail.com>
