# TreeSummarizedExperiment

[![codecov](https://codecov.io/gh/fionarhuang/TreeSummarizedExperiment/branch/master/graph/badge.svg)](https://codecov.io/gh/fionarhuang/TreeSummarizedExperiment)
[![R-CMD-check](https://github.com/fionarhuang/TreeSummarizedExperiment/workflows/R-CMD-check/badge.svg?branch=CI-1)](https://github.com/fionarhuang/TreeSummarizedExperiment/actions)

# Installation

`BiocManager::install("TreeSummarizedExperiment")`

# The structure

![](https://github.com/fionarhuang/TreeSummarizedExperiment/blob/master/vignettes/tse.png)

# [TreeSummarizedExperiment in F1000article](https://f1000research.com/articles/9-1246)
