#' The \code{TreeSummarizedExperiment} package
#'
#' \code{TreeSummarizedExperiment} implement a class of the same name, which
#' extends SingleCellExperiment to include hierarchical information on the rows
#' or columns of the rectangular data.
#'
#' It also includes an additional slot for storing reference sequences per
#' feature.
#'
#' @name TreeeSummarizedExperiment-package
#' @docType package
#' @seealso
#' \link[=TreeSummarizedExperiment-class]{TreeSummarizedExperiment} class
NULL

#' @import methods
NULL

#' TreeSummarizedExperiment internals
#'
#' Internal functions which should only be used in TreeSummarizedExperiment
#'
#' @keywords internal
#'
#' @name TreeSummarizedExperiment-internal
NULL
