% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal_utils.R
\name{.all_nonnull_in_list}
\alias{.all_nonnull_in_list}
\title{all elements in the list are NULL}
\usage{
.all_nonnull_in_list(x)
}
\description{
all elements in the list are NULL
}
\keyword{internal}
