% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal_utils.R
\name{.any_null_in_list}
\alias{.any_null_in_list}
\title{Any element in the list is NULL}
\usage{
.any_null_in_list(x)
}
\description{
Any element in the list is NULL
}
\keyword{internal}
