% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal_utils.R
\name{.rbind_refSeq}
\alias{.rbind_refSeq}
\title{rbind referenceSeq}
\usage{
.rbind_refSeq(args)
}
\description{
rbind referenceSeq
}
\keyword{internal}
