% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal_utils.R
\name{.match_phylo}
\alias{.match_phylo}
\title{match a phylo to a list of phylo}
\usage{
.match_phylo(phy, phys)
}
\description{
match a phylo to a list of phylo
}
\keyword{internal}
